@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2014 by Barry A. Warsaw
+# Copyright (C) 2004-2015 Barry Warsaw
 #
 # This file is part of flufl.enum
 #
@@ -13,6 +13,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with flufl.enum.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Author: Barry Warsaw <barry@python.org>
 
 """Package init."""
 
@@ -21,12 +23,13 @@ from __future__ import absolute_import, print_function, unicode_literals
 __metaclass__ = type
 __all__ = [
     'Enum',
+    'EnumValue',
     'IntEnum',
     '__version__',
     'make',
     ]
 
 
-__version__ = '4.0.1'
+__version__ = '4.1'
 
-from ._enum import Enum, IntEnum, make
+from ._enum import Enum, EnumValue, IntEnum, make
