@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 by Barry A. Warsaw
+# Copyright (C) 2011-2015 Barry Warsaw
 #
 # This file is part of flufl.enum.
 #
@@ -14,6 +14,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with flufl.enum.  If not, see <http://www.gnu.org/licenses/>.
+#
+# Author: Barry Warsaw <barry@python.org>
 
 """Additional package tests."""
 
@@ -36,7 +38,6 @@ from operator import attrgetter, index
 SPACE = ' '
 
 
-
 class TestBasic(unittest.TestCase):
     """Basic functionality tests"""
 
@@ -70,14 +71,9 @@ class TestBasic(unittest.TestCase):
         for c in self.all_colors:
             self.assertEqual(str(Color[c]), 'Color.' + c)
 
-    def test_enum_members(self):
-        self.assertEqual(
-            [str(m) for m in self.Colors.__members__],
-            self.all_colors)
-        for func in (str, repr):
-            self.assertEqual(
-                func(self.Colors),
-                '<Colors {red: 1, green: 2, blue: 3}>')
+    def test_enum_dir(self):
+        # dir() returns the list of enumeration item names.
+        self.assertEqual(sorted(dir(self.Colors)), sorted(self.all_colors))
 
     def test_enumclass_getitem(self):
         self.assertEqual(self.Colors[2], self.Colors.green)
@@ -85,9 +81,14 @@ class TestBasic(unittest.TestCase):
         self.assertEqual(self.Colors[self.Colors.red], self.Colors.red)
 
     def test_iteration(self):
-        self.assertEqual(
-            [v.name for v in self.Colors],
-            self.all_colors)
+        # Iteration over basic Enums is by attribute name.
+        A = Enum('A', 'a b c')
+        self.assertEqual(list(v.name for v in A), ['a', 'b', 'c'])
+        B = Enum('B', 'c b a')
+        self.assertEqual(list(v.name for v in B), ['a', 'b', 'c'])
+        # If iteration sorted over values, this would give a TypeError.
+        C = Enum('C', dict(a='7', b=7))
+        self.assertEqual(list(v.name for v in C), ['a', 'b'])
 
     def test_hashing(self):
         getvalue = attrgetter('value')
@@ -119,23 +120,23 @@ class TestBasic(unittest.TestCase):
             self.assertIsNot(first, second)
             self.assertNotEqual(first, second)
 
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.red < self.Colors.blue
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.red <= self.Colors.blue
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.red > self.Colors.green
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.green >= self.Colors.blue
 
     def test_comparison_with_int(self):
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.red < 3
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.red <= 3
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.blue > 2
-        with self.assertRaises(NotImplementedError):
+        with self.assertRaises(TypeError):
             self.Colors.green >= 1
 
         self.assertNotEqual(self.Colors.green, 2)
@@ -164,20 +165,19 @@ class TestBasic(unittest.TestCase):
         from pickle import dumps, loads
         self.assertIs(Fruit.tomato, loads(dumps(Fruit.tomato)))
 
-    def test_functional_api(self):
-        # single string
+    def test_functional_api_single_string(self):
         animals = Enum('Animals', 'ant bee cat dog')
         self.assertEqual(
             repr(animals),
             '<Animals {ant: 1, bee: 2, cat: 3, dog: 4}>')
 
-        # sequence
+    def test_functional_api_sequence(self):
         people = Enum('People', ('anne', 'bart', 'cate', 'dave'))
         self.assertEqual(
             repr(people),
             '<People {anne: 1, bart: 2, cate: 3, dave: 4}>')
 
-        # sequence of 2-tuples
+    def test_functional_api_2_tuples(self):
         def enumiter():
             start = 1
             while True:
@@ -188,11 +188,12 @@ class TestBasic(unittest.TestCase):
             repr(flags),
             '<Flags {a: 1, b: 2, c: 4, d: 8, e: 16, f: 32, g: 64}>')
 
-        # key->value dicts. Note: repr is sorted by value
+    def test_functional_api_dict(self):
+        # Note: repr is sorted by attribute name
         bassists = dict(geddy='rush', chris='yes', flea='rhcp', jack='cream')
         self.assertEqual(
             repr(Enum('Bassists', bassists)),
-            '<Bassists {jack: cream, flea: rhcp, geddy: rush, chris: yes}>')
+            '<Bassists {chris: yes, flea: rhcp, geddy: rush, jack: cream}>')
 
     def test_invalid_getitem_arguments(self):
         # getitem on an Enum with invalid value raises an exception.
@@ -212,7 +213,9 @@ class TestBasic(unittest.TestCase):
                 kyle = 3
                 kenny = 3 # Oops!
                 butters = 4
-        self.assertEqual(str(cm.exception), 'Multiple enum values: 3')
+        self.assertEqual(
+            str(cm.exception),
+            "Conflicting enum value '3' for names: 'kenny' and 'kyle'")
 
     def test_no_duplicates_in_subclass(self):
         class Colors(Enum):
@@ -223,12 +226,16 @@ class TestBasic(unittest.TestCase):
             class MoreColors(Colors):
                 yellow = 4
                 magenta = 2 # Oops!
-        self.assertEqual(str(cm.exception), 'Multiple enum values: 2')
+        self.assertEqual(
+            str(cm.exception),
+            "Conflicting enum value '2' for names: 'green' and 'magenta'")
 
     def test_no_duplicates_in_dict(self):
         with self.assertRaises(ValueError) as cm:
             Enum('Things', dict(a='yes', b='no', c='maybe', d='yes'))
-        self.assertEqual(cm.exception.args[0], 'Multiple enum values: yes')
+        self.assertEqual(
+            cm.exception.args[0],
+            "Conflicting enum value 'yes' for names: 'a' and 'd'")
 
     def test_functional_api_not_all_2_tuples(self):
         # If 2-tuples are used, all items must be 2-tuples.
@@ -260,6 +267,15 @@ class TestBasic(unittest.TestCase):
             Enum('Foo', (('ant', 1), ('bee', 2), ('3', 'cat')))
         self.assertEqual(cm.exception.args[0], 'non-identifiers: 3')
 
+    def test_functional_api_identifiers_lp1167052(self):
+        # LP: #1167052
+        self.assertRaises(ValueError, Enum, 'X', 'a-1')
+
+    def test_functional_api_identifiers_numbers(self):
+        # There was a typo in IDENTIFIER_RE where the range 0-0 was used.
+        MyEnum = Enum('X', 'a9')
+        self.assertEqual(MyEnum.a9.name, 'a9')
+
     def test_deprecate_make(self):
         # LP: #1162375 -- use Enum() calling syntax instead.
         with warnings.catch_warnings(record=True) as seen:
@@ -271,8 +287,34 @@ class TestBasic(unittest.TestCase):
         # We don't need to assert the deprecation message.
         self.assertEqual(Animals.ant.value, 1)
 
+    def test_deprecate_getitem_by_name(self):
+        # LP: #1167091
+        # Enum[item_name] should be deprecated.
+        Fruit = Enum('Fruit', dict(
+            apple='red', banana='yellow', tangerine='orange',
+            orange='reddish yellow'))
+        with warnings.catch_warnings(record=True) as seen:
+            # In Python 3.3+ we can use self.assertWarns()
+            warnings.simplefilter('always')
+            item = Fruit['banana']
+        self.assertEqual(len(seen), 1)
+        self.assertEqual(seen[0].category, DeprecationWarning)
+        # But for now it still works.
+        self.assertIs(item, Fruit.banana)
+        # getitem() should still access by value, without deprecation warning.
+        with warnings.catch_warnings(record=True) as seen:
+            # In Python 3.3+ we can use self.assertWarns()
+            warnings.simplefilter('always')
+            item = Fruit['yellow']
+        self.assertEqual(len(seen), 0)
+        self.assertIs(item, Fruit.banana)
+
+    def test_explicit_getattr(self):
+        Fruit = Enum('Fruit', 'apple banana tangerine orange')
+        self.assertIs(getattr(Fruit, 'banana'), Fruit.banana)
+        self.assertIs(getattr(Fruit, Fruit.banana.name), Fruit.banana)
+
 
-
 class TestIntEnum(unittest.TestCase):
     """Tests of the IntEnums."""
 
@@ -320,3 +362,11 @@ class TestIntEnum(unittest.TestCase):
             anchovies = 1
             black_olives = 2
         self.assertEqual(self.Animals.bee, Toppings.black_olives)
+
+    def test_iteration(self):
+        # Iteration over IntEnums is by value.
+        A = IntEnum('A', 'a b c')
+        self.assertEqual(list(v.name for v in A), ['a', 'b', 'c'])
+        B = IntEnum('B', 'c b a')
+        # Iteration over this enum is different than if it were an Enum.
+        self.assertEqual(list(v.name for v in B), ['c', 'b', 'a'])
